#include "uie_albumart.h"

#ifdef ENABLE_MATROSKA
DECLARE_COMPONENT_VERSION("Album Art Panel feat. Matroska","0.1512","A Columns_UI extension by Nathan Kallus.\r\nModified by David L to fix redrawing issues and add animation\r\nPorted to Foobar2000 0.9 by G-Lite")
#else
DECLARE_COMPONENT_VERSION("Album Art Panel","0.151","A Columns_UI extension by Nathan Kallus.\r\nModified by David L to fix redrawing issues and add animation\r\nPorted to Foobar2000 0.9 by G-Lite")
#endif

// {5549FA34-673C-4c5b-B6CE-5CDC4813E7B4}
const GUID albumart_control_base::class_guid =
{ 0x5549fa34, 0x673c, 0x4c5b, { 0xb6, 0xce, 0x5c, 0xdc, 0x48, 0x13, 0xe7, 0xb4 } };

uie_albumart::uie_albumart() {
	animshouldstart=animstarted=lastwidth=lastheight=0;
	bufnew=bufold=NULL;
	animating=false;
	b_dblclick = false;
	m_wnd = NULL;
#ifdef ENABLE_MATROSKA
	MatroskaCleanup();
#endif
}

bool uie_albumart::is_available(const uie::window_host_ptr & p_host) const
{
	return m_host.is_empty() || p_host->get_host_guid() != m_host->get_host_guid();
}

HWND uie_albumart::create_or_transfer_window(HWND wnd_parent, const uie::window_host_ptr & p_host, const ui_helpers::window_position_t & p_position)
{
	if (m_wnd == NULL) {
		static const wchar_t g_class_name[] = L"foo_uie_albumart.dll window class";
		WNDCLASS wc;
		memset(&wc,0,sizeof(wc));
		wc.style = CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;
		wc.lpfnWndProc = host_proc;
		wc.hInstance = core_api::get_my_instance();
		wc.hCursor = LoadCursor(0,IDC_ARROW);
		wc.hbrBackground = 0;
		wc.lpszClassName = g_class_name;
		RegisterClass(&wc);
		m_wnd = CreateWindowEx(exstyle_from_config(),
			g_class_name,L"foo_uie_albumart",
			WS_CHILD,
			CW_USEDEFAULT,CW_USEDEFAULT,
			0,0,
			wnd_parent,0,core_api::get_my_instance(),this);
	} else {
		SetParent(m_wnd, wnd_parent);
		m_host->relinquish_ownership(m_wnd);
	}
	m_host = p_host;
	return m_wnd;
}

void uie_albumart::destroy_window()
{
#ifdef ENABLE_MATROSKA
	MatroskaCleanup();
#endif
	if (bufold) {
		delete bufold;
		bufold=0;
	}
	if (bufnew) {
		delete bufnew;
		bufnew=0;
	}
	if (bufanim) {
		delete bufanim;
		bufanim=0;
	}
	set_handle(seltrack, 0);
	set_handle(plytrack, 0);
	set_handle(crntrack, 0);
	GdiplusShutdown(gdiplusToken);
	if (m_wnd != NULL) {
		//foo_uie_albumart_var_notify.get_static_instance().remove_window(m_wnd);
		foo_uie_albumart_playlist_notify.get_static_instance().remove_window(m_wnd);
		foo_uie_albumart_playback_notify.get_static_instance().remove_window(m_wnd);
		DestroyWindow(m_wnd);
		m_wnd = NULL;
	}
	m_host.release();
}

LRESULT WINAPI uie_albumart::host_proc(HWND wnd1,UINT msg,WPARAM wp,LPARAM lp){
	uie_albumart * p_this;
	
	if(msg == WM_NCCREATE)
	{
		p_this = (uie_albumart *)((CREATESTRUCT *)(lp))->lpCreateParams; //retrieve pointer to class
		uSetWindowLong(wnd1, GWL_USERDATA, (LPARAM)p_this);//store it for future use
	}else{
		p_this = reinterpret_cast<uie_albumart*>(uGetWindowLong(wnd1,GWL_USERDATA));//if isnt wm_create, retrieve pointer to class
	}
	return p_this ? p_this->on_message(wnd1, msg, wp, lp) : uDefWindowProc(wnd1, msg, wp, lp);
}

void uie_albumart::set_anim_timer() {
	SetTimer(m_wnd,7,10,NULL);
}

LRESULT uie_albumart::on_message(HWND wnd1,UINT msg,WPARAM wp,LPARAM lp) {
	switch(msg)
	{
	case WM_CREATE:
		GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);

		//foo_uie_albumart_var_notify.get_static_instance().add_window(wnd1);
		foo_uie_albumart_playlist_notify.get_static_instance().add_window(wnd1);
		foo_uie_albumart_playback_notify.get_static_instance().add_window(wnd1);

		image_locked=false;
		crnsource=nextgoodsource=0;
		if(plytrack.is_empty()){
			metadb_handle_ptr track;
			static_api_ptr_t<playback_control>()->get_now_playing(track);
			if(track.is_valid())
				set_handle(plytrack,track);
		}
		if(seltrack.is_empty()){
			metadb_handle_ptr track;
			static_api_ptr_t<playlist_manager>()->activeplaylist_get_focus_item_handle(track);
			if(track.is_valid())
				set_handle(seltrack,track);
		}
		if(img.is_empty() && set_image())
			redraw();
		break;
	case WM_LBUTTONDBLCLK:
		click_func(cfg_dblclickfunc);
		b_dblclick = true;
		break;
	case WM_LBUTTONUP:
		if(!b_dblclick) {  
		click_func(cfg_lftclickfunc);
		}
		b_dblclick=false;
		break;
	case WM_MBUTTONUP:
		click_func(cfg_mdlclickfunc);
		break;
	case WM_PAINT:
		paint();
		return 0;
		break;
	case WM_TIMER:
		if (wp==7) {
		KillTimer(m_wnd,7);
		paint();
		return 0;
		}
		break;
	case WM_GETMINMAXINFO:
		{
			LPMINMAXINFO mmi = LPMINMAXINFO(lp);
			mmi->ptMinTrackSize.y = cfg_minheight;
		}
		break;
	case WM_FOO_CALLBACK:
		switch(wp)
		{
		case PLAYBACK_NEW_TRACK:
			{
				metadb_handle * p_track = reinterpret_cast<metadb_handle*>(lp);
				metadb_handle_ptr track(p_track);
				if (track.is_valid())
					p_track->service_release();
				set_handle(plytrack,track);
				if(!cfg_selected && !image_locked && set_image(plytrack))
					redraw();
			}
			break;
		case PLAYBACK_STOP:
			if(lp!=playback_control::stop_reason_starting_another && !image_locked && seltrack.is_valid() && set_image(seltrack))
				redraw();
			break;
		case PLAYLIST_FOCUS_CHANGE:
			{
				metadb_handle_ptr track;
				static_api_ptr_t<playlist_manager>()->activeplaylist_get_item_handle(track, (t_size)lp);
				if(track.is_valid())
					set_handle(seltrack,track);
			}
			if (cfg_selected || !static_api_ptr_t<playback_control>()->is_playing()){
				if(seltrack.is_valid())
				{
					if(!image_locked && set_image(seltrack))
						redraw();
				}
			}
			break;
		case VARIABLE_CHANGE:
			{
				GUID * guid = reinterpret_cast<GUID *>(lp);
				if(*guid == cfg_minheight_guid){
					m_host->on_size_limit_change(wnd1,uie::size_limit_minimum_height);
				}else if(*guid == cfg_usecontrol_guid){
					lock(false);
					if(set_image())
						redraw();
				}else if(*guid == cfg_src_list_guid){
					crnsource=0;
				}
			}
			if(!image_locked)
				set_image();
			redraw();
			break;
		}
		break;
	}
	return uDefWindowProc(wnd1, msg, wp, lp);
}

class albumart_menu_functions : public uie::menu_node_command_t
{
private:
	int func_idx;
	cfg_bool * checked;
public:
	bool get_display_data(pfc::string_base & p_out,unsigned & p_state) const
	{
		p_out = func_name[func_idx];
		if (checked != NULL && *checked)
			p_state = state_checked;
		else
			p_state = 0;
		return true;
	}
	bool get_description(pfc::string_base & p_out) const
	{
		return false;
	}
	virtual void execute()
	{
		foo_uie_albumart.get_static_instance().click_func(func_idx);
	}

	albumart_menu_functions(int p_func_idx, cfg_bool * p_checked = NULL)
	{
		func_idx = p_func_idx;
		checked = p_checked;
	}
};

class albumart_menu_preferences : public uie::menu_node_command_t
{
public:
	bool get_display_data(pfc::string_base & p_out,unsigned & p_state) const
	{
		p_out = "Preferences...";
		p_state = 0;
		return true;
	}
	bool get_description(pfc::string_base & p_out) const
	{
		return false;
	}
	virtual void execute()
	{
		static_api_ptr_t<ui_control>()->show_preferences(albumart_preferences_guid);
	}
};

void uie_albumart::get_menu_items (uie::menu_hook_t & p_hook)
{
	uie::menu_node_ptr item;
	for(int i=1;i<tabsize(func_name);i++)
	{
		if (i == FUNC_FOLLOW)
			item = new albumart_menu_functions(i, &cfg_selected);
		else
			item = new albumart_menu_functions(i);
		p_hook.add_node(item);
	}
	item = new uie::menu_node_separator_t();
	p_hook.add_node(item);
	item = new albumart_menu_preferences();
	p_hook.add_node(item);
}

void uie_albumart::redraw()
{
	RedrawWindow(m_wnd, NULL, NULL, RDW_INVALIDATE | RDW_ERASE);
}

void uie_albumart::click_func(unsigned func)
{
	switch(func){
	case FUNC_OPENEX:
		func_OpenEx();
		break;
	case FUNC_OPENDIR:
		func_OpenDir();
		break;
	case FUNC_FOLLOW:
		func_ToggleFollow();
		break;
	case FUNC_FOCUSNP:
		func_FocusPlaying();
		break;
	case FUNC_REFRESH:
		func_Refresh();
		break;
	case FUNC_NEXTSRC:
		func_NextSource();
		break;
	case FUNC_RESETSRC:
		func_ResetSource();
		break;
	}
}

void uie_albumart::func_OpenEx()
{
	if(!img.is_empty())
		uShellExecute(0,"open",img,0,0,SW_SHOW);
}

void uie_albumart::func_OpenDir()
{
	if(crntrack.is_valid()){
		string8 temp;
		crntrack->format_title(NULL,temp,"$replace(%_path%,%_filename_ext%,)",NULL);
		uShellExecute(0,"open",temp,0,0,SW_SHOW);
	}
}

void uie_albumart::func_ToggleFollow()
{
	cfg_selected=!cfg_selected;
}

void uie_albumart::func_FocusPlaying()
{
	static_api_ptr_t<playlist_manager>()->highlight_playing_item();
}

void uie_albumart::func_Refresh()
{
	if(!image_locked)
		set_image();
	redraw();
}

void uie_albumart::func_NextSource() //WigBaM's modified code
{
unsigned n, oldsource; // antihang; // very helfpful for testing purposes;]
oldsource = nextgoodsource; //we save index of *currently displaying* image (which is set in set_image)
if(crntrack.is_valid()){
 unsigned m = cfg_src_list.get_count();
 n=crnsource+1;
// antihang = 0; // testing
 while(1)
 {
  if(n>=m) // ATTENTION: Changed this to m (was m-1). Otherwise we would skip last entry
   n=0;
  for(;cfg_src_list[n].find_first("-")==0;n++){ // get next source
   if(n==crnsource)
    break;
   else if(n>=m-1)
    n=-1;
  }
  if(n==crnsource)  // in case we are back to where
   break;    // we've starterd

  if(set_image(crntrack,n))  // ok, so do we have _different_ image then?
  {
   crnsource=nextgoodsource; // yes, let it be our current source then
   redraw();
   break;
  }
  else if(nextgoodsource!=oldsource) // nah, image path is the same. Now let's check whether
  {         // it is from different source entry. Weird, but happens.
           // purely for additional bug support :)
   crnsource=nextgoodsource; // OH YEAH, BABY! (c) Austin + YELLO
   break;					
  }

// So far we have found the same image with same index, so let's see
// whether we've scanned every entry and returned to the initial source
  if(nextgoodsource < n)
   break;

  n=nextgoodsource+1;  // Otherwise let's see what's next

////// TESTING
/*   antihang++;
  if(antihang>50)
  {
   console::output(1, "hanged :(((");
   break;
  }//*/
 }
}
}

void uie_albumart::func_ResetSource()
{
	if(crntrack.is_valid() && set_image(crntrack,(crnsource=0)))
		redraw();
}

bool uie_albumart::set_image(){
	return set_image(!static_api_ptr_t<playback_control>()->is_playing()?seltrack:cfg_selected?seltrack:plytrack);
}

bool uie_albumart::set_image(unsigned fstsource){
	return set_image(!static_api_ptr_t<playback_control>()->is_playing()?seltrack:cfg_selected?seltrack:plytrack,fstsource);
}

bool uie_albumart::set_image(metadb_handle_ptr track){
	if(!cfg_resetsource)
		return set_image(track,crnsource);
	else{
		string8 img_org=img;
		if(set_image(track,crnsource)){
			crnsource=0;
			img=img_org;
			return set_image(track,crnsource);
		}else
			return false;
	}
}

bool uie_albumart::set_image(metadb_handle_ptr track, unsigned fstsource)
{
	unsigned n,m = cfg_src_list.get_count();
	if(track.is_empty() || fstsource>=m) return false;
	string8 img_org=img;
	img.reset();
	set_handle(crntrack,track);
	if(m==0) {
		animshouldstart=GetTickCount();
		return true;
	}
	for(n=fstsource;n<m;n++)
	{
		string8 pattern=cfg_src_list[n];
		if(pattern.find_first("-")==0)
			pattern.remove_chars(0,1);
		bool match=false;
		if(pattern.find_first("match:")==0){
			match=true;
			pattern.remove_chars(0,6);
		}
#ifdef ENABLE_MATROSKA
		bool matroska = false;
		if (pattern.find_first("matroska:")==0) {
			pattern.remove_chars(0, 9);
			string8 ext;
			m_attachment_filename.reset();
			string8 old_filepath = m_filepath;
			crntrack->format_title(NULL, m_filepath, "%_path%", NULL);
			m_fileext = string_extension(m_filepath);
			if (stricmp_utf8(m_fileext, "MKA") == 0) {
				if (stricmp_utf8(old_filepath, m_filepath) != 0) {
					MatroskaCleanup();
					service_ptr_t<file> p_file;
					filesystem::g_open_read(p_file, m_filepath, m_abort);
					m_parser = new Matroska::MatroskaAudioParser(p_file, m_abort);
					if (m_parser != NULL) {
						m_parser->Parse(true);
						if (m_parser->GetAttachmentList().size() > 0) {
							m_attachment_list = m_parser->GetAttachmentList();
							matroska = true;
						}
						delete m_parser;
						m_parser = NULL;
					}
				} else {
					matroska = true;
				}
			}
		}
		if (!matroska) MatroskaCleanup(false);
#endif
		string8 file;
		crntrack->format_title(NULL,file,pattern,NULL);
		if(match){
			file=findFirstImageMatch(file);
		}
#if ENABLE_MATROSKA
		if (matroska) {
			file = MatroskaFindFirstImageMatch(file);
		}
		if(!file.is_empty()){
			if (matroska) {
				if (m_stream == NULL) {
					m_stream = MatroskaReadAttachment(file);
				}
				Bitmap bmp(m_stream);
				if(bmp.GetHeight()){
					m_attachment_filename = file;
					file = m_filepath;
					img=file;
					nextgoodsource=n;
					break;
				}
			} else {
				Bitmap bmp(stringcvt::string_wide_from_utf8(file.get_ptr()));
				if(bmp.GetHeight()){
					MatroskaCleanup();
					m_attachment_filename.reset();
					m_filepath.reset();
					img=file;
					nextgoodsource=n;
					break;
				}
			}
#else
		if(!file.is_empty()){
			Bitmap bmp(stringcvt::string_wide_from_utf8(file.get_ptr()));
			if(bmp.GetHeight()){
				 img=file;
				 nextgoodsource=n;
				 break;
			}
#endif
		}
		if(n==fstsource-1 || (n==m-1 && fstsource==0)) break;
		if(n==m-1) n=-1;
	}
	animshouldstart=GetTickCount();
	return img.is_empty() ? !img_org.is_empty() : strcmp_partial(img_org,img)!=0 ;
}

void uie_albumart::paint()
{
	RECT wndrect_temp;
	if(GetClientRect(m_wnd,&wndrect_temp)){
		Graphics gfx(m_wnd);
		Rect wndrect(
			wndrect_temp.left,
			wndrect_temp.top,
			wndrect_temp.right-wndrect_temp.left,
			wndrect_temp.bottom-wndrect_temp.top
			);
		
		
		bool noimage=(!bufnew);
		bool imagechanged=(strcmp(bufimg.get_ptr(),img.get_ptr())!=0);
		bool sizechanged=bufnew?(bufnew->GetWidth()!=wndrect.Width||bufnew->GetHeight()!=wndrect.Height):true;
		if (noimage||imagechanged||sizechanged||configchanged) {
			
			SetWindowLong(m_wnd, GWL_EXSTYLE, exstyle_from_config());
			SetWindowPos(m_wnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOZORDER|SWP_FRAMECHANGED);

			if (sizechanged||noimage) {
				if (animating) animating=false;
				if (bufold) {
					delete bufold;
					bufold=NULL;
				}
				if (bufanim) {
					delete bufanim;
					bufanim=NULL;
				}
				if (bufnew) {
					delete bufnew;
					bufnew=NULL;
				}
			bufnew = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
			bufold = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
			if (cfg_animtime!=0)
				bufanim = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
			} else if (configchanged) {
				if (bufanim&&cfg_animtime==0) {
					delete bufanim;
					bufanim=NULL;
				} else if (!bufanim&&cfg_animtime!=0) {
					bufanim = new Bitmap(wndrect.Width, wndrect.Height, &gfx);
				}
			}
				
			
			
		if (configchanged) configchanged=false;
				Bitmap *temp;
		temp=bufold;
		bufold=bufnew;
		bufnew=temp;
		
				Graphics bufgfx(bufnew);
		bufgfx.Clear(cfg_usebg?Color((cfg_bgcol) & 0xFF,(cfg_bgcol>> 8) & 0xFF,(cfg_bgcol>> 16) & 0xFF):Color::Black);

		if(!img.is_empty()){
#ifdef ENABLE_MATROSKA
			Bitmap* bmp;
			if (!m_attachment_filename.is_empty()) {
				m_stream = MatroskaReadAttachment(m_attachment_filename);
				bmp = new Bitmap(m_stream);
			} else {
				bmp = new Bitmap(stringcvt::string_wide_from_utf8(img.get_ptr()));
			}
#else
			Bitmap bmp(stringcvt::string_wide_from_utf8(img.get_ptr()));
#endif
		
			
			/*
			if(cfg_padding>0){
				wndrect.Height-=2*cfg_padding;
				wndrect.Width-=2*cfg_padding;
				wndrect.X+=cfg_padding;
				wndrect.Y+=cfg_padding;
			}
			*/
#ifdef ENABLE_MATROSKA
			RectF rect(wndrect.X,wndrect.Y,bmp->GetWidth(),bmp->GetHeight());
#else
			RectF rect(wndrect.X,wndrect.Y,bmp.GetWidth(),bmp.GetHeight());
#endif
			if(
				(cfg_expandalbum && rect.GetRight() < wndrect.GetRight()  &&  rect.GetBottom() < wndrect.GetBottom())  ||
				(cfg_shrinkalbum  &&  (rect.GetRight() > wndrect.GetRight()  ||  rect.GetBottom() > wndrect.GetBottom()))){
				if(!cfg_aspectratio){
					rect.Width=wndrect.GetRight();
					rect.Height=wndrect.GetBottom();
				}
				else if( (wndrect.GetRight()-rect.GetRight()) < (wndrect.GetBottom()-rect.GetBottom()) ){
					rect.Height*=((double)wndrect.GetRight()/rect.GetRight());
					rect.Width=wndrect.GetRight();
				}else{
					rect.Width*=((double)wndrect.GetBottom()/rect.GetBottom());
					rect.Height=wndrect.GetBottom();
				}
			}
			if(cfg_centeralbum)
				rect.Offset((double)(wndrect.GetRight()-rect.GetRight())/2,(double)(wndrect.GetBottom()-rect.GetBottom())/2);
			if(cfg_pixelborder){
				rect.Width--;
				rect.Height--;
				bufgfx.DrawRectangle(& Pen(Color(0,0,0)),rect);
				rect.X++;
				rect.Y++;
				rect.Width--;
				rect.Height--;
			}
			bufgfx.SetInterpolationMode(cfg_interpolationmode==2?InterpolationModeHighQuality:
					cfg_interpolationmode==1?InterpolationModeDefault:
					cfg_interpolationmode==0?InterpolationModeLowQuality:
					InterpolationModeHighQualityBicubic);
#ifdef ENABLE_MATROSKA
			int srcWidth=bmp->GetWidth(),srcHeight=bmp->GetHeight(),srcX=0,srcY=0;
#else
			int srcWidth=bmp.GetWidth(),srcHeight=bmp.GetHeight(),srcX=0,srcY=0;
#endif
			/*
			if(cfg_padding<0 && abs(cfg_padding)<bmp.GetWidth() && abs(cfg_padding)<bmp.GetHeight()){
				srcWidth+=2*cfg_padding;
				srcHeight+=2*cfg_padding;
				srcX-=cfg_padding;
				srcY-=cfg_padding;
			}
			*/
			
#ifdef ENABLE_MATROSKA
			bufgfx.DrawImage(
				bmp,
				rect,  // destination rectangle 
				srcX, srcY,        // upper-left corner of source rectangle
				srcWidth,       // width of source rectangle
				srcHeight,      // height of source rectangle
				UnitPixel);
			delete bmp;
			bmp = NULL;
#else
			bufgfx.DrawImage(
				&bmp,
				rect,  // destination rectangle 
				srcX, srcY,        // upper-left corner of source rectangle
				srcWidth,       // width of source rectangle
				srcHeight,      // height of source rectangle
				UnitPixel);
#endif
		}
				bufimg=img;
		
				// bufnew updated. time to fade in the changes.
				DWORD now=GetTickCount();
				if (cfg_animtime!=0&&!sizechanged&&(animshouldstart+700)>now) {
				animating=true;
				animstarted=GetTickCount();
				set_anim_timer();
				}
		}
		
		
			
		Rect dest(wndrect_temp.left,
			wndrect_temp.top,
			wndrect_temp.right-wndrect_temp.left,
			wndrect_temp.bottom-wndrect_temp.top);
			if (bufnew&&!animating) {
					gfx.DrawImage(bufnew, 0, 0);
			} else if (animating&&bufnew&&bufanim) {
				float opacityold=0.0f;
				float opacitynew=1.0f;
				int doneness=GetTickCount();
				if (doneness>(animstarted+cfg_animtime)) {
				animating=false;
				opacityold=0.0f;
				opacitynew=1.0f;
				} else if (animstarted<=doneness&&cfg_animtime)
				{
					opacitynew=(float)(doneness-animstarted)/(float)cfg_animtime;
					opacityold=1.0f-0.0f;
				}
				{
					Graphics animbuf(bufanim);
					animbuf.Clear(cfg_usebg?Color((cfg_bgcol) & 0xFF,(cfg_bgcol>> 8) & 0xFF,(cfg_bgcol>> 16) & 0xFF):Color::Black);
					ColorMatrix cm;
			ZeroMemory(&cm,sizeof(ColorMatrix));
			cm.m[0][0] = 1.0f;
			cm.m[1][1] = 1.0f;
			cm.m[2][2] = 1.0f;
			cm.m[3][3] = opacityold;

			ImageAttributes ia;
			ia.SetColorMatrix(&cm);
			animbuf.DrawImage(bufold,dest,0,0,dest.Width,dest.Height,UnitPixel,&ia);
			cm.m[3][3] = opacitynew;
			ia.SetColorMatrix(&cm);
			animbuf.DrawImage(bufnew,dest,0,0,dest.Width,dest.Height,UnitPixel,&ia);
				}
				gfx.DrawImage(bufanim,0,0);
			if (animating)
			set_anim_timer();
			}
			else gfx.Clear(cfg_usebg?Color((cfg_bgcol) & 0xFF,(cfg_bgcol>> 8) & 0xFF,(cfg_bgcol>> 16) & 0xFF):Color::Black);

		ValidateRect(m_wnd,NULL);

		}
	
}

void uie_albumart::lock(bool l){
	image_locked=l;
}

bool uie_albumart::is_locked(){
	return image_locked;
}

#ifdef ENABLE_MATROSKA
IStream* uie_albumart::MatroskaReadAttachment(string8 filename) {
	for (int i = 0; i != m_attachment_list.size(); ++i) {
		if (stricmp_utf8(stringcvt::string_utf8_from_wide(m_attachment_list.at(i).FileName.c_str()), filename) == 0) {
			Matroska::MatroskaAttachment attachment = m_attachment_list.at(i);
			service_ptr_t<file> p_file;
			filesystem::g_open_read(p_file, m_filepath, m_abort);
			if (p_file == NULL) return NULL;
			p_file->seek(attachment.SourceStartPos, m_abort);
			HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE, attachment.SourceDataLength);
			if (hGlobal == NULL) return NULL;
			m_buffer = GlobalLock(hGlobal);
			if (m_buffer == NULL) {
				GlobalFree(hGlobal);
				return NULL;
			}
			p_file->read(m_buffer, attachment.SourceDataLength, m_abort);
			GlobalUnlock(hGlobal);
			IStream* stream;
			HRESULT hr = CreateStreamOnHGlobal(hGlobal, true, &stream);
			if (SUCCEEDED(hr) == FALSE) {
				stream->Release();
				stream = NULL;
				return NULL;
			}
			return stream;
		}
	}
	return NULL;
}

string8 uie_albumart::MatroskaFindFirstImageMatch(string8 p)
{
	for (int i = 0; i != m_attachment_list.size(); ++i) {
		string8 name = stringcvt::string_utf8_from_wide(m_attachment_list.at(i).FileName.c_str());
		if (wildcard_helper::has_wildcards(p)) {
			if (wildcard_helper::test(name, p)) {
				return name;
			}
		} else {
			if (stricmp_utf8(name, p) == 0) {
				return name;
			}
		}
	}
	return NULL;
}

void uie_albumart::MatroskaCleanup(bool p_all)
{
	if (m_file != NULL && p_all) {
		m_file = NULL;
	}
	if (m_parser != NULL && p_all) {
		delete m_parser;
		m_parser = NULL;
	}
	if (!m_attachment_list.empty() && p_all) {
		m_attachment_list.clear();
	}
	if (m_stream != NULL) {
		m_stream->Release();
		m_stream = NULL;
	}
	if (m_buffer != NULL) {
		m_buffer = NULL;
	}
}
#endif
